//
//    Modifications and changes to the original Anchor Chips software
//	  are Copyright 2004, Thomas C. McDermott, N5EG. Only the modifications
//	  and changes constitute the GPL licensed software.
//
//    This file is part of VNAR - the Vector Network Analyzer program.
//
//    VNAR is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    VNAR is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with VNAR, if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//	  Version 1.0 target application code - VNA for Cypress AN2135 target.
//
//	  4-01-04  TCM	Version 1.0
//	  4-26-04  TCM  Version 1.0.1 - Add measurement integration option: 1x, 2x, 4x, 8x, 16x
//	  6-04-04  TCM  Version 1.1.0 - Add FAST measurement mode (collect 5 readings per USB packet)
//   10-29-04  TCM  Version 2.0.0 - Add High and Low readings of S21M to both SLOW and FAST routines
//								  - This changes the packet response formats, and requires storing
//								  - IDAC Hi and Low settings given to us by the host.
//   03-18-06  TCM  Version 3.0.0 - Remove Integration Count. Add MeasureSettleTime for Slow reads.
//								  - Also Add Mid reading of S21M to only the SLOW routine.
//	 09-30-07  TCM  Version 3.1.0 - Add Trans Phase Lo (I & Q) readings to the FULL response.
//    							  - Expand reading settling delay to 16-bits for 30 & 100 msec delays.




#pragma NOIV					// Do not generate interrupt vectors
//-----------------------------------------------------------------------------
//	File:		periph.c
//	Contents:	Hooks required to implement USB peripheral function.
//
//	Copyright (c) 1997 AnchorChips, Inc. All rights reserved
//-----------------------------------------------------------------------------
#include "ezusb.h"
#include "ezregs.h"

extern BOOL	GotSUD;			// Received setup data flag
extern BOOL	Sleep;
extern BOOL	Rwuen;
extern BOOL	Selfpwr;

BYTE	Configuration;		// Current configuration
BYTE	AlternateSetting;	// Alternate settings

// ---------------------------------------------------------------------------
// internal function prototypes
// ---------------------------------------------------------------------------

void ADC_Read(WORD *dat);
void ADC_Read1(WORD *dat);
void ADC_Write(BYTE dat);
void ADC_Write1(BYTE dat);		// for testing IIC errors
//BYTE DDS_Read(BYTE addr);
void DDS_Write(BYTE addr, BYTE dat);

// ----------------------------------------------------------------------------
// our global variables
// ----------------------------------------------------------------------------

int	exhaust;					// for DDS test routines
int	in2owner;					// for arming IN2EP
BOOL ScanNeeded;				// Flag set when Host has requested a data set
BYTE ADC_Status;				// ADC Errors BIT7=TX_ERR BIT6=RX_ERR
WORD MeasDelay;					// Measurement Delay (in loop counts)
int TxType;						// Requested Tx buffer Format
unsigned int IDACHigh, IDACLo, IDACMid; // Host-sent values for High, Mid, and Low DAC levels
unsigned int IDACPhLo;			// Host-sent value for Lo Phase readings
								// Measurements
WORD	S11MIR, S11PIR, VREF1I, S21MIR, S21PIR, VREF2I;
WORD	S11MIF, S11PIF;
WORD	S11MQR, S11PQR, VREF1Q, S21MQR, S21PQR, VREF2Q;
WORD	S11MQF, S11PQF; 
WORD	S21MQFLo, S21MQFMid;
WORD    S21PIFLo, S21PQFLo;


	// 4 additional TX frequency control words (48 bits each)

BYTE	TX_FREQ_MSB0[4], TX_FREQ_MSB1[4], TX_FREQ_MSB2[4];
BYTE	TX_FREQ_MSB3[4], TX_FREQ_MSB4[4], TX_FREQ_MSB5[4];


//-----------------------------------------------------------------------------
// Task Dispatcher hooks
//	The following hooks are called by the task dispatcher.
//-----------------------------------------------------------------------------

void TD_Init(void) 				// Called once at startup
{

	int i;

	exhaust = 1000;				// number of poll cycles to wait for DDS to wake up
//	in2owner = 0;				// input 2 owned by polling loop (=1 is by interrupt loop)
	Rwuen = TRUE;				// Enable remote-wakeup
	ScanNeeded = FALSE;

// Initialization routines for the Vector Network Analyzer

	CPUCS |= bmCLK24OE;			// enable 24 Mhz output buffer
	OEC = 0xff;					// enable PC7:PC0 outputs
	USBPAIR = 0;				// Don't pair any endpoints

	PORTACFG = 0;				// PA4 and PA5 are port pins
	OEA = 0x30;					// PA4 and PA5 are outputs
	OUTA = 0;					// Set PA4 and PA5 low

//	ISODISAB |= bmBIT0;			// disable ISOCHRONOUS transfers (freeing memory)

// Enable endpoints 2 IN and OUT
	IN07VAL |= bmEP2;
	OUT07VAL |= bmEP2;

// Enable interrupts for endpoints 2 IN and OUT 
	OUT07IEN |= bmEP2;
	IN07IEN |= bmEP2;

// Have some known bad data ready for the first Host Request

	IN2BUF[0] = 0xFF;

	for (i=1; i<64; i++)
		IN2BUF[i] = 0x0;
    IN2CS = 0;					// clear STALL
//	IN2BC = 64;					// Arm the input buffer



	ADC_Write(0x0D2);				// SETUP ADC - Internal Clock
//	ADC_Write(0xDA);				// Setup ADC - External Clock (IIC SCL)


}

void TD_Poll(void) 				// Called repeatedly while the device is idle
{

//	int a;
	int del;					// delay loop for read, write
	int j;

	WORD Result;

	// The DDS wakes up later than the EZUSB processor.
	// Wait about a 1000 polling loops before initializing it.

if(exhaust > 0)
	{
		exhaust--;
		return;						// Wait for DDS to wake up
	}

if (exhaust == 0)				// DDS probably awake - initialize  it
	{
		DDS_Write(0x1D, 0x10);		// Same as default, COMPARATOR powered down
		DDS_Write(0x1E, 0x4C);		// High speed PLL, REFMULT = 12x (288 MHz)
		DDS_Write(0x1F, 0x01);		// Same as default, CW output

		DDS_Write(0x20, 0x40);		// Bypass invsinc, no shaping of the outputs

//		DDS_Write(0x20, 0x60);		// Bypass invsinc, enable I and Q DAC level controls

		DDS_Write(0x21, 0x0F);		// I-DAC MSB = maximum  (4095 decimal is max)
		DDS_Write(0x22, 0xFF);		// I-DAC LSB = maximum
		DDS_Write(0x23, 0x0F);		// Q-DAC MSB = maximum  (4095 decimal is max)
		DDS_Write(0x24, 0xFF);		// Q-DAC LSB = maximum

		DDS_Write(0x4, 0x08);		// DDS synthesizer wakes up
		DDS_Write(0x5, 0x0E3);		// at a default value of 10.0 MHz

		exhaust--;					// set it to -1 so this initialization code only gets called once
	}

//if (!in2owner)						// if polling loop owns IN2
//	{
//	    if (!IN2CS)					// if IN2 isn't busy - it should be
//		{
//			IN2CS = 0;				// clear stall
//			IN2BC = 64;
//		}
//	}

if (ScanNeeded)			// If scan needed, build a response packet for the host
	{
		
//		if( IN2CS & bmEPBUSY)	// If the buffer is busy, don't fill it
//			return;			// It will get filled on next POLL when empty

		DDS_Write(0x20, 0x60);			// turn on I and Q DAC multipliers in DDS chip (save power)

		if (TxType == 0)	// FULL response type
		{
	
			// Delay between Frequency change and read (SLOW type)

			for(del=0; del<MeasDelay; del++)   
			{
				ADC_Write(0x67);			// dummy just to add some time delay
	  			ADC_Read(&Result);			// dummy just to add some time delay
			}


//			ADC_Write(0x0D2);				// SETUP ADC	

			ADC_Status = 0;					// Clear I2C error bits

		

// FORMAT of the Type 0 FULL Response:
//
//  Offset  Word (LSB first, MSB second)		Condition
//  ------  -------------------------------     --------------
//	0000    0100 								Valid frame
//  0002    S11M S11P VREF1 S21MHi S21P VREF2 	Ref=I, TX=REFL
//  000E    S11M S11P							Ref=I, TX=FWD
//  0012    S11M S11P VREF1 S21MHi S21P VREF2	Ref=Q, TX=REFL
//  001E    S11M S11P							Ref=Q, TX=FWD
//  0022    S21MLo								Ref=Q, TX=FWD
//  0024	S21MMid								Ref=Q, TX=FWD
//	0026    S21PILo								Ref=I, TX=FWD
//  0028    S21PQLo								Ref=Q, TX=FWD
//  002A	remainder is 00

				OUTA = 0;						// PA4 = 0, PA5 = 0 -> Ref=I, TX=REFL

		
				ADC_Write(0x61); 			// request ADC measure chan 0
			  	ADC_Read(&Result);				// 12-bit result, unsigned int format
				S11MIR = Result;
	
				ADC_Write(0x63);			// request ADC measure chan 1
			  	ADC_Read(&Result);				// 12-bit result, unsigned int format
				S11PIR = Result;
	
				ADC_Write(0x65); 			// request ADC measure chan 2
			  	ADC_Read(&Result);				// 12-bit result, unsigned int format
				VREF1I = Result;
	
				ADC_Write(0x67); 			// request ADC measure chan 3
			  	ADC_Read(&Result);				// 12-bit result, unsigned int format
				S21MIR = Result;
	
				ADC_Write(0x69);			// request ADC measure chan 4
			  	ADC_Read(&Result);				// 12-bit result, unsigned int format
				S21PIR = Result;
	
				ADC_Write(0x6B); 			// request ADC measure chan 5
			  	ADC_Read(&Result);				// 12-bit result, unsigned int format
				VREF2I = Result;
	
	
	
				OUTA = 0x10;					// PA5 = 0 PA4 = 1 -> Ref=I, TX=FWD
		
	   	  		ADC_Write(0x61);			// request ADC measure chan 0
		  		ADC_Read(&Result);			// 12-bit result, unsigned int format
				S11MIF = Result;
	
	   	  		ADC_Write(0x63);				// request ADC measure chan 1
		  		ADC_Read(&Result);			// 12-bit result, unsigned int format
				S11PIF = Result;
		
		
				OUTA = 0x20;					// PA5 = 1 PA4 = 0 -> Ref=Q, Tx=REFL
		
				ADC_Write(0x61);			// request ADC measure chan 0
			  	ADC_Read(&Result);				// 12-bit result, unsigned int format
				S11MQR = Result;
	
				ADC_Write(0x63); 			// request ADC measure chan 1
			  	ADC_Read(&Result);				// 12-bit result, unsigned int format
				S11PQR = Result;

				ADC_Write(0x65); 			// request ADC measure chan 2
			  	ADC_Read(&Result);				// 12-bit result, unsigned int format
				VREF1Q = Result;
	
				ADC_Write(0x67); 			// request ADC measure chan 3
			  	ADC_Read(&Result);				// 12-bit result, unsigned int format
				S21MQR = Result;
	
				ADC_Write(0x69); 			// request ADC measure chan 4
			  	ADC_Read(&Result);				// 12-bit result, unsigned int format
				S21PQR = Result;
	
				ADC_Write(0x6B); 			// request ADC measure chan 5
			  	ADC_Read(&Result);				// 12-bit result, unsigned int format
				VREF2Q = Result;
		
		
				OUTA = 0x30;					// PA5 = 1 PA4 = 1 -> Ref=Q, TX=FWD
		
				ADC_Write(0x61);			// request ADC measure chan 0
		  		ADC_Read(&Result);			// 12-bit result, unsigned int format
				S11MQF = Result;
	
	 	  		ADC_Write(0x63);			// request ADC measure chan 1
		  		ADC_Read(&Result);			// 12-bit result, unsigned int format
				S11PQF = Result;

		//  Measure S21M with Mid output transmit level

				// set output transmit level to low

				DDS_Write(0x21, MSB(IDACMid));	// I-DAC MSB = Mid level 
				DDS_Write(0x22, LSB(IDACMid));	// I-DAC LSB = Mid level

				for(del=0; del<MeasDelay; del++)   // delay between level change and read
				{
	
					ADC_Write(0x67);			// dummy just to add some time delay
		  			ADC_Read(&Result);			// dummy just to add some time delay
				}

				ADC_Write(0x67);			// request ADC measure chan 3
		  		ADC_Read(&Result);			// 12-bit result, unsigned int format
				S21MQFMid = Result;



		//  Measure S21P with Low output transmit level (09-30-2007)

				// set output transmit level to phase level low

				DDS_Write(0x21, MSB(IDACPhLo));	// I-DAC MSB = Phase-Low level 
				DDS_Write(0x22, LSB(IDACPhLo));	// I-DAC LSB = Phase-Low level

				for(del=0; del<MeasDelay; del++)   // delay between level change and read
				{
	
					ADC_Write(0x67);			// dummy just to add some time delay
		  			ADC_Read(&Result);			// dummy just to add some time delay
				}


				ADC_Write(0x69);				// request ADC measure chan 1
			  	ADC_Read(&Result);				// 12-bit result, unsigned int format
				S21PQFLo = Result;


				OUTA = 0x10;					// PA5 = 0 PA4 = 1 -> Ref=I, TX=FWD

				ADC_Write(0x69);				// request ADC measure chan 1
			  	ADC_Read(&Result);				// 12-bit result, unsigned int format
				S21PIFLo = Result;



		//  Measure S21M with Low output transmit level

				// set output transmit level to low

				DDS_Write(0x21, MSB(IDACLo));	// I-DAC MSB = maximum 
				DDS_Write(0x22, LSB(IDACLo));	// I-DAC LSB = maximum

				OUTA = 0x30;					// PA5 = 1 PA4 = 1 -> Ref=Q, TX=FWD

				for(del=0; del<MeasDelay; del++)   // delay between level change and read
				{
	
					ADC_Write(0x67);			// dummy just to add some time delay
		  			ADC_Read(&Result);			// dummy just to add some time delay
				}

				ADC_Write(0x67);			// request ADC measure chan 3
		  		ADC_Read(&Result);			// 12-bit result, unsigned int format
				S21MQFLo = Result;

				// restore output transmit level back to high

				DDS_Write(0x21, MSB(IDACHigh));	// I-DAC MSB = maximum 
				DDS_Write(0x22, LSB(IDACHigh));	// I-DAC LSB = maximum



	
			IN2BUF[2] = LSB(S11MIR);	IN2BUF[3] = MSB(S11MIR);
			IN2BUF[4] = LSB(S11PIR);	IN2BUF[5] = MSB(S11PIR);
			IN2BUF[6] = LSB(VREF1I);	IN2BUF[7] = MSB(VREF1I);
			IN2BUF[8] = LSB(S21MIR);	IN2BUF[9] = MSB(S21MIR);
			IN2BUF[10] = LSB(S21PIR);	IN2BUF[11] = MSB(S21PIR);
			IN2BUF[12] = LSB(VREF2I);	IN2BUF[13] = MSB(VREF2I);
	
			IN2BUF[14] = LSB(S11MIF);	IN2BUF[15] = MSB(S11MIF);
			IN2BUF[16] = LSB(S11PIF);	IN2BUF[17] = MSB(S11PIF);
	
			IN2BUF[18] = LSB(S11MQR);	IN2BUF[19] = MSB(S11MQR);
			IN2BUF[20] = LSB(S11PQR);	IN2BUF[21] = MSB(S11PQR);
			IN2BUF[22] = LSB(VREF1Q);	IN2BUF[23] = MSB(VREF1Q);
			IN2BUF[24] = LSB(S21MQR);	IN2BUF[25] = MSB(S21MQR);
			IN2BUF[26] = LSB(S21PQR);	IN2BUF[27] = MSB(S21PQR);
			IN2BUF[28] = LSB(VREF2Q);	IN2BUF[29] = MSB(VREF2Q);
			
			IN2BUF[30] = LSB(S11MQF);	IN2BUF[31] = MSB(S11MQF);
			IN2BUF[32] = LSB(S11PQF);	IN2BUF[33] = MSB(S11PQF);
			IN2BUF[34] = LSB(S21MQFLo); IN2BUF[35] = MSB(S21MQFLo);
			IN2BUF[36] = LSB(S21MQFMid); IN2BUF[37] = MSB(S21MQFMid);
			IN2BUF[38] = LSB(S21PIFLo); IN2BUF[39] = MSB(S21PIFLo);	
			IN2BUF[40] = LSB(S21PQFLo); IN2BUF[41] = MSB(S21PQFLo);	

			OUTA = 0;						// PA4, PA5 both zero at end of loop

		} // end if FULL reponse type



		if (TxType == 1)				// FAST Response type
		{

//			ADC_Write(0x0D2);				// SETUP ADC

			ADC_Status = 0;					// Clear I2C error bits


// FORMAT of the Type 1 FAST Response (for VNAR3):
//
//  Offset  Word (LSB first, MSB second)							Condition
//  ------  -------------------------------     					--------------
//	0000    0100 													Valid frame
//  0002    S11MQ S11PI S11PQ S21MQHi S21PI S21PQ S21MQLo S21MQMid 	Freq1
//  0012    S11MQ S11PI S11PQ S21MQHi S21PI S21PQ S21MQLo S21MQMid	Freq2
//  0022    S11MQ S11PI S11PQ S21MQHi S21PI S21PQ S21MQLo S21MQMid	Freq3
//  0032    S11MQ S11PI S11PQ S21MQHi S21PIMix S21PQMix S21MQLoMix	Freq4							
//
//  Mix values contain Mixed returns. The low-12 bits of each is the S21PI,
//	S21PQ, and S21MQLo values for the 4th reading. The upper 4-bits of each,
//  comprise the 12 bits of S21MQMid which needs to borrow the buffer space.
//

			for (j=0; j<4; j++)				// perform readings at 4 frequencies
											// DDS already programmed at Freq 1
			{

				OUTA = 0x0;						// PA4 = 0 PA5 = 0 -> Ref=I, TX=REFL


				ADC_Write(0x63);			// dummy just to add some time delay
	  			ADC_Read(&Result);			// dummy just to add some time delay

				ADC_Write(0x63);			// request ADC measure chan 1
			  	ADC_Read(&Result);				// 12-bit result, unsigned int format
				IN2BUF[j*0x10 + 4]= LSB(Result);	// S11PI
				IN2BUF[j*0x10 + 5]= MSB(Result);


				ADC_Write(0x69); 			// request ADC measure chan 4
			  	ADC_Read(&Result);				// 12-bit result, unsigned int format
				IN2BUF[j*0x10 + 0x0A]= LSB(Result);	// S21PI
				IN2BUF[j*0x10 + 0x0B]= MSB(Result);


				OUTA = 0x20;					// PA5 = 1 PA4 = 0 -> Ref=Q, Tx=REFL


				ADC_Write(0x61); 			// request ADC measure chan 0
			  	ADC_Read(&Result);				// 12-bit result, unsigned int format
				IN2BUF[j*0x10 + 2]= LSB(Result);	// S11MQ
				IN2BUF[j*0x10 + 3]= MSB(Result);

	
				ADC_Write(0x63); 			// request ADC measure chan 1
			  	ADC_Read(&Result);				// 12-bit result, unsigned int format
				IN2BUF[j*0x10 + 6]= LSB(Result);	// S11PQ
				IN2BUF[j*0x10 + 7]= MSB(Result);

				ADC_Write(0x67); 			// request ADC measure chan 3
			  	ADC_Read(&Result);				// 12-bit result, unsigned int format
				IN2BUF[j*0x10 + 8]= LSB(Result);	// S21MQHi
				IN2BUF[j*0x10 + 9]= MSB(Result);

				ADC_Write(0x69); 			// request ADC measure chan 4
			  	ADC_Read(&Result);				// 12-bit result, unsigned int format
				IN2BUF[j*0x10 + 0x0C]= LSB(Result);	// S21PQ
				IN2BUF[j*0x10 + 0x0D]= MSB(Result);

		// Program DAC level to Lo and re-run S21MQLo

				DDS_Write(0x21, MSB(IDACLo));	// I-DAC MSB = maximum 
				DDS_Write(0x22, LSB(IDACLo));	// I-DAC LSB = maximum

				ADC_Write(0x67);			// dummy just to add some time delay
	  			ADC_Read(&Result);			// dummy just to add some time delay

				ADC_Write(0x67); 			// request ADC measure chan 3
			  	ADC_Read(&Result);				// 12-bit result, unsigned int format
				IN2BUF[j*0x10 + 0x0E]= LSB(Result);	// S21MQLo
				IN2BUF[j*0x10 + 0x0F]= MSB(Result);


		// Program DAC level to Mid and re-run S21MQMid

				DDS_Write(0x21, MSB(IDACMid));	// I-DAC MSB = maximum 
				DDS_Write(0x22, LSB(IDACMid));	// I-DAC LSB = maximum

				ADC_Write(0x67);			// dummy just to add some time delay
	  			ADC_Read(&Result);			// dummy just to add some time delay

				ADC_Write(0x67); 			// request ADC measure chan 3
			  	ADC_Read(&Result);				// 12-bit result, unsigned int format

				if (j<3)					// Mid value for first three results
				{
					IN2BUF[j*0x10 + 0x10]= LSB(Result);	// S21MQMid
					IN2BUF[j*0x10 + 0x11]= MSB(Result);
				}
				else						// Mid value is multiplexed into top 4 bits of 3 existing values
				{
					IN2BUF[0x3B] |= ((MSB(Result) & 0x0F) << 4);  // pack MS nibble into S21PIMix top 4 bits
					IN2BUF[0x3D] |= (LSB(Result) & 0xF0);  // pack 2nd nibble into S21PQMix top 4 bits
					IN2BUF[0x3F] |= ((LSB(Result) & 0x0F) << 4); // pack LS nibble into S21MQLoMix top 4 bits
				}


		// Program DAC level back to Hi

				DDS_Write(0x21, MSB(IDACHigh));	// I-DAC MSB = maximum 
				DDS_Write(0x22, LSB(IDACHigh));	// I-DAC LSB = maximum


				if (j==3)							// don't attempt to program DDS at Freq 5
					break;
													// Program DDS to next frequency
				DDS_Write(0x4, TX_FREQ_MSB0[j]);		// MS byte of Freq divisor
				DDS_Write(0x5, TX_FREQ_MSB1[j]);		//    byte of Freq divisor
				DDS_Write(0x6, TX_FREQ_MSB2[j]);		//    byte of Freq divisor
				DDS_Write(0x7, TX_FREQ_MSB3[j]);		//    byte of Freq divisor
				DDS_Write(0x8, TX_FREQ_MSB4[j]);		//    byte of Freq divisor
				DDS_Write(0x9, TX_FREQ_MSB5[j]);		// LS byte of Freq divisor

			}// end for(4 readings)

			OUTA = 0x30;					// return to Ref=Q, Tx=Fwd mode
		}// end if FAST response type



		IN2BUF[1] = ADC_Status;			// 00 = No I2C errors
		IN2CS = 0;						// clear stall
		ScanNeeded = FALSE;				// Scan now valid - don't reply with dummies
		IN2BUF[0] = 0x01;				// 01 = Valid Scan buffer
		IN2BC = 64;						// arm input buffer - so host can read our result
//		OUT2BC = 0;						// arm output buffer

		DDS_Write(0x20, 0x40);			// turn off I and Q DAC multipliers in DDS chip (save power)

	} // end if(scan_needed)


}	//end TD_Poll


// Since VNA is self-powered, suspend and resume do not have any meaning
// but will leave the interrupt handlers here just in case the host tries
// to suspend,resume us.

BOOL TD_Suspend(void) 			// Called before the device goes into suspend mode
{
	return(TRUE);
}

BOOL TD_Resume(void) 			// Called after the device resumes
{
	return(TRUE);
}

//-----------------------------------------------------------------------------
// Device Request hooks
//	The following hooks are called by the end point 0 device request parser.
//-----------------------------------------------------------------------------

BOOL DR_GetDescriptor(void)
{
	return(TRUE);
}

BOOL DR_SetConfiguration(void)	// Called when a Set Configuration command is received
{
	Configuration = SETUPDAT[2];
	return(TRUE);				// Handled by user code
}

BOOL DR_GetConfiguration(void)	// Called when a Get Configuration command is received
{
	IN0BUF[0] = Configuration;
	EZUSB_SET_EP_BYTES(IN0BUF_ID,1);
	return(TRUE);				// Handled by user code
}

BOOL DR_SetInterface(void) 		// Called when a Set Interface command is received
{
	AlternateSetting = SETUPDAT[2];
	return(TRUE);				// Handled by user code
}

BOOL DR_GetInterface(void) 		// Called when a Set Interface command is received
{
	IN0BUF[0] = AlternateSetting;
	EZUSB_SET_EP_BYTES(IN0BUF_ID,1);
	return(TRUE);				// Handled by user code
}

BOOL DR_GetStatus(void)
{
	return(TRUE);
}

BOOL DR_ClearFeature(void)
{
	return(TRUE);
}

BOOL DR_SetFeature(void)
{
	return(TRUE);
}

BOOL DR_VendorCmnd(void)
{
	return(TRUE);
}

//-----------------------------------------------------------------------------
// USB Interrupt Handlers
//	The following functions are called by the USB interrupt jump table.
//-----------------------------------------------------------------------------

// Setup Data Available Interrupt Handler
void ISR_Sudav(void) interrupt 0
{
	GotSUD = TRUE;				// Set flag
	EZUSB_IRQ_CLEAR();
	USBIRQ = bmSUDAV;			// Clear SUDAV IRQ
}

// Setup Token Interrupt Handler
void ISR_Sutok(void) interrupt 0
{
	EZUSB_IRQ_CLEAR();
	USBIRQ = bmSUTOK;			// Clear SUTOK IRQ
}
// SOF happens every 1 millisecond
void ISR_Sof(void) interrupt 0
{
	EZUSB_IRQ_CLEAR();
	USBIRQ = bmSOF;				// Clear SOF IRQ
}
// USB Bus Reset detected
void ISR_Ures(void) interrupt 0
{

    EPIO[OUT2BUF_ID].bytes = 0;	// Arm EP2 out so it can receive from host

	EZUSB_IRQ_CLEAR();
	USBIRQ = bmURES;			// Clear URES IRQ
}

void ISR_IBN(void) interrupt 0
{
}

void ISR_Susp(void) interrupt 0
{
	Sleep = TRUE;
	EZUSB_IRQ_CLEAR();
	USBIRQ = bmSUSP;
}

void ISR_Ep0in(void) interrupt 0
{
}

void ISR_Ep0out(void) interrupt 0
{
}

void ISR_Ep1in(void) interrupt 0
{
}

void ISR_Ep1out(void) interrupt 0
{
}

// This IN interrupt occurs on completion of IN2BUF transfer to
// the host, signifying that IN2BUF is now empty and can be reloaded.
// IN2BUF is the 64-byte buffer, IN2BC is the byte count.
// Loading IN2BC auto-arms the buffer, and sets the BUSY flag.
// Sucessful ACK by the host of the transfer clears the BUSY flag
// (and generates this interrupt).
// EZUSB will NAK the host on it's attempt to read an un-armed buffer.
// Note the EP1-7 are UNARMed by a processor reset, and any read by the
// host will be NAKed until they are explicitly ARMed by us.
void ISR_Ep2in(void) interrupt 0
{
//	in2owner = 1;				// interrupt routine now owns IN2

//	if(!ScanNeeded)				// arm buffer if no scan is pending
//	{
//	    IN2BUF[0] = 0xFE;			// empty frame type
//	    IN2BUF[1] = 0x00;			// no IIC errors
//		IN2CS = 0;					// clear STALL
//		IN2BC = 64;					// buffer is ready - arm it.
//	}

// New to Proto8 - arm the output buffer once we finish sending to the host
    OUT2BC = 0;

    // clear the IRQ
	EZUSB_IRQ_CLEAR();
	IN07IRQ = bmEP2;

}

// This OUT interrupt occurs when the host has completed a transfer to us.
// OUT2BUF is the buffer, OUT2BC is the byte count, and BUSY is set.
// Writing OUT2BC with any value re-arms the buffer, and clears the BUSY flag.
// EZUSB will NAK the host on it's attempt to write to a still-busy buffer.
// Note that EP1-7 are ARMed by a processor reset, and is able to accept
// one buffer immediately.
void ISR_Ep2out(void) interrupt 0
{
	// Load the DDS synthesizer with the 6-byte frequency from the packet
	// OUT2BUF is an __int64 type - 8 bytes. We care about the lower 6 bytes.
	// It is sent to us reversed:
	// OUT2BUF[0] = LSByte, OUT2BUF[7] = MSByte

	BYTE i;
	for (i=0; i<6; i++)					// use lower 48 bits of value
	{
		DDS_Write((0x4+i), OUT2BUF[5-i]);
	}

	// Set the DAC output I- and Q- levels.
	// Ref Level comes as a 16 bit unsigned int, byte-reversed in the buffer

	DDS_Write(0x21, OUT2BUF[9]);	// I-DACHigh MSB
	DDS_Write(0x22, OUT2BUF[8]);	// I-DACHigh LSB
	DDS_Write(0x23, OUT2BUF[11]);	// Q-DAC MSB
	DDS_Write(0x24, OUT2BUF[10]);	// Q-DAC MSB		

	// Record the TxType format and Measurement Delay requested

	TxType = (unsigned int)OUT2BUF[12];		// LSB goes to TxType (MSB set to zero)
	MeasDelay = (unsigned int)OUT2BUF[14] + ((unsigned int)OUT2BUF[15] << 8); // MeasDelay is byte-reversed in buffer 

	// Record the IDACHigh, IDAClo, and IDACMid level values.

	IDACHigh = (unsigned int)OUT2BUF[8] + ((unsigned int)OUT2BUF[9] << 8);	// IDACHigh is byte-reversed in buffer 
	IDACLo  = (unsigned int)OUT2BUF[48] + ((unsigned int)OUT2BUF[49] << 8);	// IDACLo is byte-reversed in buffer
	IDACMid = (unsigned int)OUT2BUF[50] + ((unsigned int)OUT2BUF[51] << 8); // IDACMid is byte-reversed in buffer

	// Record the IDACPhLo level value (09-30-2007)

	IDACPhLo = (unsigned int)OUT2BUF[52] + ((unsigned int)OUT2BUF[53] << 8); // IDACPhLo is byte-reversed in buffer

	if (TxType == 1)			// FAST buffer response type
	{
		for (i=2; i<5; i++)		// Save remaining 3 frequencies to be measured in FAST mode
		{
			TX_FREQ_MSB0[i-2] = OUT2BUF[i*8 + 5];	// Most significant byte
			TX_FREQ_MSB1[i-2] = OUT2BUF[i*8 + 4];
			TX_FREQ_MSB2[i-2] = OUT2BUF[i*8 + 3];
			TX_FREQ_MSB3[i-2] = OUT2BUF[i*8 + 2];
			TX_FREQ_MSB4[i-2] = OUT2BUF[i*8 + 1];
			TX_FREQ_MSB5[i-2] = OUT2BUF[i*8 + 0];	// Least significant byte
		}
	};			

	ScanNeeded = TRUE;	// Need to Scan ADC and build packet for host

	// Proto 8 - don't rearm the OUT buffer. Wait until host has acquired
	// our response buffer, then rearm.
	// OUT2BC = 0;			// Release the buffer, rearm the output

    // clear the IRQ
	EZUSB_IRQ_CLEAR();
	OUT07IRQ = bmEP2;
}

void ISR_Ep3in(void) interrupt 0
{
}

void ISR_Ep3out(void) interrupt 0
{
}

void ISR_Ep4in(void) interrupt 0
{
}

void ISR_Ep4out(void) interrupt 0
{
}

void ISR_Ep5in(void) interrupt 0
{
}

void ISR_Ep5out(void) interrupt 0
{
}

void ISR_Ep6in(void) interrupt 0
{
}

void ISR_Ep6out(void) interrupt 0
{
}

void ISR_Ep7in(void) interrupt 0
{
}

void ISR_Ep7out(void) interrupt 0
{
}


//-----------------------------------------------------------------------------
// Added Support Routines for the VNA project.
//	Write to I2C ADC and Read from I2C ADC
//	The ADC has an I2C bus address of 6Ah for writes and 06Bh for reads
//-----------------------------------------------------------------------------

void ADC_Write(BYTE dat)	// version to retry the write upon error
{
	ADC_Write1(dat);
	while(ADC_Status & (bmBIT7 | bmBIT6))	// if BIT 6 or 7 error is set
	{
		ADC_Status &= ~(bmBIT7 | bmBIT6);	// clear bit 7 and 6
		ADC_Write1(dat);		// and retry the write
	}
}

void ADC_Write1(BYTE dat)	// write byte to ADC via I2C port
{
	BYTE status;

	while (I2CS & bmSTOP)	// wait until STOP is low
		;

	I2CS = bmSTART;		// set START bit
	I2DAT = 0x6A;		// ADC address + Write
	while (!(I2CS & bmDONE))
		 ; 		// wait for done bit

	status = (I2CS &(bmBERR | bmACK)); // get status register
	if (status != bmACK)
	{
		I2CS = bmSTOP;	// stop the I2C bus
		ADC_Status |= bmBIT7;
		return;	// I2C bus error
	}
	I2DAT = dat;		// write byte to I2C device
	while (!(I2CS & bmDONE))
		 ; 		// wait for done bit

	status = (I2CS &(bmBERR | bmACK)); // get status register
	if (status != bmACK)
	{
		I2CS = bmSTOP;
		ADC_Status |= bmBIT6;
		return;	// I2C bus error
	}

	I2CS = bmSTOP;		// stop the I2C bus
	return;
}



void ADC_Read(WORD *dat)	// version to retry the write upon error
{
	ADC_Read1(dat);
	while(ADC_Status & (bmBIT3 | bmBIT2 | bmBIT1))	// if BIT 3 or 2 error is set
	{
		ADC_Status &= ~(bmBIT3 | bmBIT2 | bmBIT1);	// clear bit 3, 2, and 1
		ADC_Read1(dat);			// and retry the read
	}
}
void ADC_Read1(WORD *dat)	// read data from ADC via I2C port
{
	BYTE status, dummy, low, high;

	while (I2CS & bmSTOP)	// wait until STOP is low
		;

	I2CS = bmSTART;		// set START bit
	I2DAT = 0x6B;		// ADC address + Read
	while (!(I2CS & bmDONE))
		 ; 		// wait for done bit

	status = (I2CS &(bmBERR | bmACK)); // get status register
	if (status != bmACK)
	{
		I2CS = bmSTOP;	// stop the I2C bus
		ADC_Status |= bmBIT3;
		return;	// I2C bus error
	}

	dummy = I2DAT;		// trigger the read

	while (!(I2CS & bmDONE))
		 ; 		// wait for done bit

	if (I2CS & bmBERR)	// if I2C Bus Error
	{
		I2CS = bmSTOP;
		ADC_Status |= bmBIT2;
		return;
	}

	I2CS = bmLASTRD;	// last read comes after trigger
	high = I2DAT;		// get high byte of data
				// and trigger last read
	while (!(I2CS & bmDONE))
		 ; 		// wait for done bit

	if (I2CS & bmBERR)	// if I2C Bus Error
	{
		I2CS = bmSTOP;
		ADC_Status |= bmBIT1;
		return;
	}

	I2CS = bmSTOP;		// don't trigger another read
	low = I2DAT;

	high &= 0x0F;		// 4 MS bits of high aren't present
	*dat = ((high << 8) + low);

	return;
}

// --------------------------------------------------------------------
// Routine to write and read the DDS synthesizer registers via the
// the databus, using PC5:0 as the address bus
// --------------------------------------------------------------------

xdata volatile BYTE dds	_at_	0x2000;	// force external memory access for dds

void DDS_Write(BYTE addr, BYTE dat)	// Write dat to the 6-bit DDS addr
{
	OUTC = addr;			// PC7-PC6 are ignored
	dds = dat;
	return;
}

// Nothing currently needs to read the DDS registers. This code is commented
// out to supress Linker Warnings (uncalled subroutine).

//BYTE DDS_Read(BYTE addr)	// Read dat from the 6-bit DDS addr
//{
//	OUTC = addr;			// PC7-PC6 are ignored
//	return(dds);
//}


